@@ -1,7 +1,8 @@
 /* PSPP - computes sample statistics.
    Copyright (C) 1997-9, 2000 Free Software Foundation, Inc.
    Written by Ben Pfaff <blp@gnu.org>.
-   Modified for R foreign library by Saikat DebRoy <saikat@stat.wisc.edu>.
+   Modified for R foreign package by Saikat DebRoy <saikat@stat.wisc.edu>
+   Patches by the R Core Team 2001-2017.
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License as
@@ -403,24 +404,39 @@ sfm_read_dictionary (struct file_handle * h, struct sfm_read_info * inf)
 
 	    switch (data.subtype)
 	      {
-	      case 3:
+			  
+		/* subtypes as specified by PSPP documentation at
+			  https://www.gnu.org/software/pspp/pspp-dev/html_node/System-File-Format.html#System-File-Format
+		*/
+			  
+	      case 3: /* Machine integer information */
 		  if (!read_machine_int32_info (h, data.size, data.count,
 						&(inf->encoding)))
 		  goto lossage;
 		break;
 
-	      case 4:
+	      case 4: /* Machine floating point information */
 		if (!read_machine_flt64_info (h, data.size, data.count))
 		  goto lossage;
 		break;
 
-	      case 5:
-	      case 6:
-	      case 11: /* ?? Used by SPSS 8.0. */
+	      case 5:  /* grouped variables */ 
 		skip = 1;
 		break;
 
-	      case 7: /* Multiple-response sets (later versions of SPSS). */
+	      case 6: /* some rarely used date information */
+		skip = 1;
+		break;
+
+	      case 7: /* Multiple-response sets */
+		skip = 1;
+		break;
+
+	      case 11: /*Variable Display Parameter Record */
+		skip = 1;
+		break;
+
+	      case 12: /* UUID, rarely used */
 		skip = 1;
 		break;
 
@@ -430,14 +446,45 @@ sfm_read_dictionary (struct file_handle * h, struct sfm_read_info * inf)
 		  goto lossage;
 		break;
 
-	      case 16: /* See http://www.nabble.com/problem-loading-SPSS-15.0-save-files-t2726500.html */
+	      case 14: /* Very Long String Record */
+		warning(_("%s: Very long string record(s) found (record type 7, subtype %d), each will be imported in consecutive separate variables"), h->fn, data.subtype);
 		skip = 1;
 		break;
 
-	      case 20:
+	      case 16: /* Extended Number of Cases Record (64-bit integer to represent ncases rather than 32-bit in system file header) */
+		  skip = 1;
+		break;
+
+	      case 17: /* Data File Attributes Record */
+		skip = 1;
+		break;
+		  
+	      case 18: /* Variable Attributes Records */
 		skip = 1;
 		break;
 
+	      case 19: /* bninary multiple-response sets */
+		skip = 1;
+		break;
+		  
+	      case 20: /* another codepage information */
+		skip = 1;
+		break;
+
+	      case 21: /* Long String Value Labels Record */
+		warning(_("%s: Long string value labels record found (record type 7, subtype %d), but ignored"), h->fn, data.subtype);
+		skip = 1;
+		break;
+
+	      case 22: /* Long String Missing Values Record */
+		warning(_("%s: Long string missing values record found (record type 7, subtype %d), but ignored"), h->fn, data.subtype);
+		skip = 1;
+		break;
+		  
+	      case 24: /* XML that describes how data in the file should be displayed on-screen */
+		skip = 1;
+		break;
+						  
 	      default:
 		warning(_("%s: Unrecognized record type 7, subtype %d encountered in system file"), h->fn, data.subtype);
 		skip = 1;
@@ -631,7 +678,8 @@ read_long_var_names (struct file_handle * h, struct dictionary * dict
       /* now, p is key, val is long name */
       for (lp = dict->var; lp < end; ++lp) {
         if (!strcmp(lp[0]->name, p)) {
-          strncpy(lp[0]->name, val, sizeof(lp[0]->name));
+          strncpy(lp[0]->name, val, 64);
+	  lp[0]->name[64] = '\0';
           break;
         }
       }
@@ -1051,9 +1099,15 @@ read_variables (struct file_handle * h, struct variable *** var_by_index)
       else
 	vv->miss_type = MISSING_NONE;
 
-      if (!parse_format_spec (h, sv.print, &vv->print, vv)
-	  || !parse_format_spec (h, sv.write, &vv->write, vv))
+      if (!parse_format_spec (h, sv.print, &vv->print, vv))
 	goto lossage;
+/*	We do not need this part in the if() condition:
+	  || !parse_format_spec (h, sv.write, &vv->write, vv))  
+	 as this produces some uncertainty about the format type given 
+	  https://www.gnu.org/software/pspp/pspp-dev/html_node/Variable-Record.html#Variable-Record
+	  tells us on June 9, 2017:
+	  "A few system files have been observed in the wild with invalid write fields, in particular with value 0. Readers should probably treat invalid print or write fields as some default format. "
+*/
     }
 
   /* Some consistency checks. */
